use tokio::{
    fs::File,
    io::{
        Error, SeekFrom,
        AsyncWriteExt, AsyncSeekExt
    },
};
use std::collections::BTreeSet;

mod header;
mod page;
mod trie;

const PAGE_SIZE :u64 = 4096;

enum Path {
    Index(usize),
    Key(&u8),
}

pub struct Database {
    file:File,
    temp:BTreeSet<u64>,
}
impl Database {
    async fn init(&mut self) -> Result<(),Error>
    {
        let mut header = header::Header::default();
        header.depth = 1;
        header.root = 2;

        let buffered = header.serialize();

        self.file.set_len(PAGE_SIZE * 3).await?;

        self.file.seek(SeekFrom::Start(0)).await?;
        self.file.write(&buffered).await?;

        self.file.seek(SeekFrom::Start(PAGE_SIZE)).await?;
        self.file.write(&buffered).await?;

        Ok(())
    }
}
impl From<File> for Database {
    fn from(value: File) -> Self
    {
        Self {
            file:value,
            temp:BTreeSet::new(),
        }
    }
}

/* Object
** Meta [16]
**  [0:3] Type
** 
**  00 Blob
**    [3:3] Size
**
**  01 List
**    []
**
**  10 Trie
**
** Addr [48]
*/

/* Page Table Entry
**  <>
*/

/* Allocation Table Entry
**  <>
**
*/

/* Header
**  <>
**  
*/
