pub enum Usage {
    Blob  = 0b00,
    List  = 0b01,
    Trie  = 0b10,
    Table = 0b11,
}

pub struct Entry {
    addr:u64,
    res:u64,
}
impl Entry {
    pub fn new() -> Self
    {
        Self {
            addr:0,
            res:0,
        }
    }

    pub fn blob(address:u64, size:u8) -> Self
    {
        let addr = address & ((1 << 48) - 1);
        let size = (size as u64) << 48;
        Self {
            addr: size|addr,
            res: 0,
        }
    }

    pub fn serialize(&self) -> [u8;16]
    {
        [
            (self.addr & 0xFF) as u8,
            ((self.addr >> 8) & 0xFF) as u8,
            ((self.addr >> 16) & 0xFF) as u8,
            ((self.addr >> 24) & 0xFF) as u8,
            ((self.addr >> 32) & 0xFF) as u8,
            ((self.addr >> 40) & 0xFF) as u8,
            ((self.addr >> 48) & 0xFF) as u8,
            ((self.addr >> 56) & 0xFF) as u8,

            (self.res & 0xFF) as u8,
            ((self.res >> 8) & 0xFF) as u8,
            ((self.res >> 16) & 0xFF) as u8,
            ((self.res >> 24) & 0xFF) as u8,
            ((self.res >> 32) & 0xFF) as u8,
            ((self.res >> 40) & 0xFF) as u8,
            ((self.res >> 48) & 0xFF) as u8,
            ((self.res >> 56) & 0xFF) as u8,
        ]
    }

    pub fn deserialize(&mut self, data:[u8;16])
    {
        self.addr = data[0] as u64
            |(data[1] << 8) as u64
            |(data[2] << 16) as u64
            |(data[3] << 24) as u64
            |(data[4] << 32) as u64
            |(data[5] << 40) as u64
            |(data[6] << 48) as u64
            |(data[7] << 56) as u64;

        self.res = data[8] as u64
            |(data[9] << 8) as u64
            |(data[10] << 16) as u64
            |(data[11] << 24) as u64
            |(data[12] << 32) as u64
            |(data[13] << 40) as u64
            |(data[14] << 48) as u64
            |(data[15] << 56) as u64;
    }
}
